; --------------------------------------------------  May 15, 2004
;|                 "AGM v1.0"                       |
;|   TI82 GRAPH_MEM line drawing routines (Ash)     |
;|            File name: "agm10.asm"                |
; --------------------------------------------------
;    -----------------------------------------
;   |Author: Jakob Selbing                    |
;   |  Comments, bugs, tips, anything on      |
;   |  your mind:                             |
;   | Email: jakse638@student.liu.se          |
;   | URL: hem.passagen.se/jaksel             |
;   |                                         |
;    -----------------------------------------
;
;
;This source is a simple TI82 program that utilises one routine:
;
; agm_bc_de    - draws a line from (b,c) to (d,e) in GRAPH_MEM
;
;
;
; The rate is about 7900 pixels / second. However if DISP_GRAPH
; is called for each line, like in this demo program, the rate 
; is about 3300 pixels / second.
;
;Note: If you want a routine that writes directly to the LCD instead, 
;      you can use the "AGM LCD" routine, available at the same place 
;      as this file hopefully.
;
;
; YOU CAN USE THESE ROUTINES FREELY, BUT PLEASE GIVE ME CREDIT. :)
;
;             Copyright (C) Jakob Selbing 2004

#include "ti82.h"
#include "keys.inc"

agm_mem_index  = TEXT_MEM      ; Memory base for variables
agm_k1         = 0
agm_k2         = 1
agm_A          = 2
agm_status     = 3

agm_code_XY_switch = 1


.org START_ADDR
.db "AGM v1.0 test",0


program_start:
    call clear_graph
    ld b, 0
    ld c, 63
    ld d, 95
    ld e, 63
testloop1:
    push bc
    push de
    call agm_bc_de
    ROM_CALL(DISP_GRAPH)
    pop de
    pop bc
    dec e
    ld a, e
    cp 255
    jr nz, testloop1
    call clear_graph

    ld b, 0
    ld c, 63
    ld d, 95
    ld e, 0
testloop2:
    push bc
    push de
    call agm_bc_de
    ROM_CALL(DISP_GRAPH)
    pop de
    pop bc
    inc b
    ld a, b
    cp 96
    jr nz, testloop2

    
    call clear_graph
    ld b, 95
    ld c, 63
    ld d, 95
    ld e, 0
testloop3:
    push bc
    push de
    call agm_bc_de
    ROM_CALL(DISP_GRAPH)
    pop de
    pop bc
    dec d
    ld a, d
    cp 255
    jr nz, testloop3

    call clear_graph
    ld b, 95
    ld c, 63
    ld d, 0
    ld e, 0
testloop4:
    push bc
    push de
    call agm_bc_de
    ROM_CALL(DISP_GRAPH)
    pop de
    pop bc
    dec c
    ld a, c
    cp 255
    jr nz, testloop4

Key_loop:
    call GET_KEY
    or a
    jr z, Key_loop
    ret
    
    
clear_graph:
    ld hl,GRAPH_MEM
    ld de,GRAPH_MEM+1
    ld (hl),0
    ld bc,767
    ldir
    ROM_CALL(CLEARLCD)
    ret



agm_bc_de:
; Draws a line from (b, c) to (d, e) in GRAPH_MEM.
; Affects registers: A, B, C, D, E, H, L
;
; b = x1, c = y1
; d = x2, e = y2
;
;        x (0..95)
;    -------->
;   |
;   |
;   |  TI82 screen
;   |
;   \/
;y (0..63)
    ld a, $8C
    out (P_ROM), a          ; Set ROM page 4 (needed for FIND_PIXEL)
    ld a, d
    sub b                   ; a = delta X
    jr nc, agm_longer_axis_skip_neg1
    neg    
agm_longer_axis_skip_neg1:
    ld h, a                 ; h = abs(delta X)
    ld a, e
    sub c                   ; a = delta Y
    jr nc, agm_longer_axis_skip_neg2
    neg
agm_longer_axis_skip_neg2:
    sub h                   ; a= abs(delta Y) - abs(delta X)
    ld a, 0                 ; Reset "switch XY" flag
    jr c, agm_longer_axis_x
    ld a, b                 ; switch X and Y
    ld b, c
    ld c, a
    ld a, d                 ; switch X and Y
    ld d, e
    ld e, a
    ld a, agm_code_XY_switch                 ; Set "switch XY" flag
agm_longer_axis_x:
    ld (agm_mem_index + agm_status), a       ; Save status
    
    ld a, b
    cp d
    jr c, agm_skip_exchange
    ld a, b                 ; Exchange bc and de
    ld b, d                 ; since algorith assumes b < d
    ld d, a
    ld a, c
    ld c, e
    ld e, a
agm_skip_exchange:

    ld a, d
    sub b
    ld (agm_mem_index + agm_k1), a        ; Save constant K1 = x2 - x1 (>= 0 always)
    ld a, e
    sub c
    ld (agm_mem_index + agm_k2), a        ; Save constant K2 = y2 - y1
    sub a                   ; A = 0
    ld (agm_mem_index + agm_A), a     ; Save variable A

agm_bc_de_loop:
    push bc
    ld a, (agm_mem_index + agm_status)
    cp agm_code_XY_switch
    jr nz, agm_no_XY_switch
    ld a, b                 ; Switch X and Y if this was done earlier
    ld b, c
    ld c, a
agm_no_XY_switch:
    push de
    ld a, 63
    sub c
    ld c, a
    call FIND_PIXEL
    ld de, GRAPH_MEM
    add hl, de
    or (hl)
    ld (hl), a
    
    pop de
    pop bc    
        
    ld a, b
    cp d                    ; Check if whole line done
    ret z                   ; Yes, quit
 
    inc b                   ; No, step forward along longer axis
    push de
    push bc

    ld a, (agm_mem_index + agm_A)     ; Load iterative value "A", which
    ld l, a                             ; should be kept as close to zero as possible
    ld h, 0
    bit 7, a
    jr z, agm_A_not_negative
    ld h, 255                           ; Extend to 16-bit 2s complement
agm_A_not_negative:

    ld a, (agm_mem_index + agm_k2)    ; Load constant K2
    ld e, a
    ld d, 0                             ; Extend to 16-bit 2s complement
    bit 7, a
    jr z, agm_K2_pos1
    ld d, 255
agm_K2_pos1:
    or a                                    ; Clear Carry flag
    sbc hl, de                              ; hl = Aold - K2 = A

    push hl
    
    ld a, (agm_mem_index + agm_k1)
    ld d, 0
    ld e, a
    ld a, (agm_mem_index + agm_k2)                    ; If delta Y negative then 
    bit 7, a                                ; we will test A - K1 instead of A + K1
    jr z, agm_K2_pos2
    or a
    sbc hl, de                              ; hl = A - K1
    jr agm_ex_de_hl
agm_K2_pos2:
    add hl, de                              ; hl = A + K1
agm_ex_de_hl:
    ex de, hl                               ; de = A +/- K1 = B
    
    pop hl                                  ; hl = A
    push hl
    push de
        
    bit 7, h
    jr z, agm_A_abs_done
    ld a, l
    neg
    ld l, a
    ld h, 0
agm_A_abs_done:
    bit 7, d
    jr z, agm_B_abs_done
    ld a, e
    neg
    ld e, a
    ld d, 0
agm_B_abs_done:
    or a                                    ; Clear Carry flag
    sbc hl, de                              ; hl = abs(A) - abs(B)
    pop de                                  ; pop B = A +/- K1
    pop hl                                  ; pop A
    pop bc                                  ; pop position 1
    jr c, agm_keep_y                       ; If abs(A) < abs(A +/- K1) then Y should not increase/decrease
    ld a, (agm_mem_index + agm_k2)
    bit 7, a
    jr nz, agm_dec_y
    inc c                                   ; Increase Y position
    inc c
agm_dec_y:
    dec c
    ld l, e                                 ; Make code below save (A +/- K1) instead of A
agm_keep_y:
    ld a, l                                 ; Keep Y position
    ld (agm_mem_index + agm_A), a         ; A' = A
    pop de                                  ; Pop position 2
    jp agm_bc_de_loop    



.end



